@@ -1,62 +1,108 @@
-package MooseX::Types::ISO8601;
-use Moose ();
+package MooseX::Types::ISO8601; # git description: v0.15-19-g0e18bc1
+# ABSTRACT: ISO8601 date and duration string type constraints and coercions for Moose
+our $VERSION = '0.16';
+use strict;
+use warnings;
+
+use utf8;
+use DateTime 0.41;
+# this alias lets us distinguish the class from the class_type in versions of
+# MooseX::Types that can't figure that out for us (i.e. before 0.32)
 use aliased DateTime => 'DT';
-use DateTime::Format::Duration;
-use MooseX::Types::DateTime qw(Duration DateTime);
+use DateTime::TimeZone;
+use DateTime::Duration;
+use DateTime::Format::Duration 1.03;
+use MooseX::Types::DateTime 0.03 qw(Duration DateTime);
 use MooseX::Types::Moose qw/Str Num/;
 use List::MoreUtils qw/ zip /;
 use Scalar::Util qw/ looks_like_number /;
-use Try::Tiny qw/try/;
+use Module::Runtime 'use_module';
+use Try::Tiny;
+use Safe::Isa;
 
 our $MYSQL;
 BEGIN {
     $MYSQL = 0;
-    if (try { Class::MOP::load_class('MooseX::Types::DateTime::MySQL') }) {
+    if (eval { require MooseX::Types::DateTime::MySQL; 1 }) {
             MooseX::Types::DateTime::MySQL->import(qw/ MySQLDateTime /);
             $MYSQL = 1;
     }
 }
-use namespace::autoclean;
+use if MooseX::Types->VERSION >= 0.42, 'namespace::autoclean';
 
-our $VERSION = "0.11";
-
-use MooseX::Types -declare => [qw(
+use MooseX::Types 0.10 -declare => [qw(
     ISO8601DateStr
     ISO8601TimeStr
     ISO8601DateTimeStr
     ISO8601DateTimeTZStr
+
+    ISO8601StrictDateStr
+    ISO8601StrictTimeStr
+    ISO8601StrictDateTimeStr
+    ISO8601StrictDateTimeTZStr
+
     ISO8601TimeDurationStr
     ISO8601DateDurationStr
     ISO8601DateTimeDurationStr
     ISO8601DateTimeDurationStr
 )];
 
-my $date_re = qr/^(\d{4})-(\d{2})-(\d{2})$/;
+my $date_re =       qr/^(\d{4})-(\d{2})-(\d{2})$/;
+my $time_re =                               qr/^(\d{2}):(\d{2}):(\d{2})(?:(?:\.|,)(\d+))?Z?$/;
+my $datetime_re =   qr/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(?:(?:\.|,)(\d+))?Z?$/;
+my $datetimetz_re = qr/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(?:(?:\.|,)(\d+))?((?:(?:\+|-)\d\d:\d\d)|Z)$/;
+
 subtype ISO8601DateStr,
     as Str,
     where { /$date_re/ };
 
-my $time_re = qr/^(\d{2}):(\d{2}):(\d{2})(?:(?:\.|,)(\d+))?Z?$/;
+# XXX TODO: this doesn't match all the ISO Time formats in the spec:
+# hhmmss
+# hhmm
+# hh
+# hh:mm
 subtype ISO8601TimeStr,
     as Str,
     where { /$time_re/ };
 
-my $datetime_re = qr/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(?:(?:\.|,)(\d+))?Z$/;
 subtype ISO8601DateTimeStr,
     as Str,
     where { /$datetime_re/ };
 
-my $datetimetz_re = qr/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(?:(?:\.|,)(\d+))?((?:\+|-)\d\d:\d\d)$/;
- subtype ISO8601DateTimeTZStr,
+# XXX TODO: this doesn't match these offset indicators:
+# ±hhmm
+# ±hh
+subtype ISO8601DateTimeTZStr,
     as Str,
     where { /$datetimetz_re/ };
 
+subtype ISO8601StrictDateStr,
+    as ISO8601DateStr,
+    where { (try { use_module('DateTime::Format::ISO8601')->parse_datetime($_) })->$_isa('DateTime') };
+
+subtype ISO8601StrictTimeStr,
+    as ISO8601TimeStr,
+    where {
+        (   try { use_module('DateTime::Format::ISO8601')->parse_datetime($_) }
+         || try { DateTime::Format::ISO8601->parse_time($_) }
+        )->$_isa('DateTime')
+    };
+
+subtype ISO8601StrictDateTimeStr,
+    as ISO8601DateTimeStr,
+    where { (try { use_module('DateTime::Format::ISO8601')->parse_datetime($_) })->$_isa('DateTime') };
+
+subtype ISO8601StrictDateTimeTZStr,
+    as ISO8601DateTimeTZStr,
+    where { (try { use_module('DateTime::Format::ISO8601')->parse_datetime($_) })->$_isa('DateTime') };
+
+
 # TODO: According to ISO 8601:2004(E), the lowest order components may be
 # omitted, if less accuracy is required.  The lowest component may also have
 # a decimal fraction.  We don't support these both together, you may only have
 # a fraction on the seconds component.
 
-my $timeduration_re = qr/^PT(?:(\d{1,2})H)?(?:(\d{1,2})M)?(?:(\d{0,2})(?:(?:\.|,)(\d+))?S)?$/;
+my $timeduration_re = qr/^PT(?:(\d+)H)?(?:(\d+)M)?(?:(\d{0,2})(?:(?:\.|,)(\d+))?S)?$/;
 subtype ISO8601TimeDurationStr,
     as Str,
     where { grep { looks_like_number($_) } /$timeduration_re/; };
@@ -66,7 +112,7 @@ subtype ISO8601DateDurationStr,
     as Str,
     where { grep { looks_like_number($_) } /$dateduration_re/ };
 
-my $datetimeduration_re = qr/^P(?:(\d+)Y)?(?:(\d{1,2})M)?(?:(\d{1,2})D)?(?:T(?:(\d{1,2})H)?(?:(\d{1,2})M)?(?:(\d{0,2})(?:(?:\.|,)(\d+))?)S)?$/;
+my $datetimeduration_re = qr/^P(?:(\d+)Y)?(?:(\d{1,2})M)?(?:(\d{1,2})D)?(?:T(?:(\d+)H)?(?:(\d+)M)?(?:(\d{0,2})(?:(?:\.|,)(\d+))?)S)?$/;
 subtype ISO8601DateTimeDurationStr,
     as Str,
     where { grep { looks_like_number($_) } /$datetimeduration_re/ };
@@ -113,6 +159,8 @@ subtype ISO8601DateTimeDurationStr,
             $_[0]->ymd('-') . 'T' . $_[0]->hms(':') . "$1:$2"
         },
     );
+    @coerce{(ISO8601StrictTimeStr, ISO8601StrictDateStr, ISO8601StrictDateTimeStr, ISO8601StrictDateTimeTZStr)} =
+        @coerce{(ISO8601TimeStr, ISO8601DateStr, ISO8601DateTimeStr, ISO8601DateTimeTZStr)};
 
     foreach my $type_name (keys %coerce) {
 
@@ -149,6 +197,8 @@ subtype ISO8601DateTimeDurationStr,
             return $_;
         },
     );
+    @coerce{(ISO8601StrictTimeStr, ISO8601StrictDateStr, ISO8601StrictDateTimeStr, ISO8601StrictDateTimeTZStr)} =
+        @coerce{(ISO8601TimeStr, ISO8601DateStr, ISO8601DateTimeStr, ISO8601DateTimeTZStr)};
 
     foreach my $type_name (keys %coerce) {
 
@@ -196,6 +246,8 @@ subtype ISO8601DateTimeDurationStr,
     coerce DateTime,
         from ISO8601DateTimeStr,
             via {
+                # TODO: surely we should be using
+                # DateTime::Format::ISO8601->parse_datetime for this
                 my @fields = map { $_ || 0 } $_ =~ /$datetime_re/;
                 if ($fields[6]) {
                     my $missing = 9 - length($fields[6]);
@@ -234,22 +286,38 @@ subtype ISO8601DateTimeDurationStr,
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 MooseX::Types::ISO8601 - ISO8601 date and duration string type constraints and coercions for Moose
 
+=head1 VERSION
+
+version 0.16
+
 =head1 SYNOPSIS
 
     use MooseX::Types::ISO8601 qw/
+        ISO8601DateTimeStr
         ISO8601TimeDurationStr
     /;
 
-    has duration => (
-        isa => ISO8601TimeDurationStr,
+    has datetime => (
         is => 'ro',
+        isa => ISO8601DateTimeStr,
+    );
+
+    has duration => (
+        is => 'ro',
+        isa => ISO8601TimeDurationStr,
         coerce => 1,
     );
 
+    Class->new( datetime => '2012-01-01T00:00:00' );
+
     Class->new( duration => 60 ); # 60s => PT00H01M00S
     Class->new( duration => DateTime::Duration->new(%args) )
 
@@ -276,6 +344,20 @@ An ISO8601 combined datetime string. E.g. C<< 2009-06-11T12:06:34Z >>
 
 An ISO8601 combined datetime string with a fully specified timezone. E.g. C<< 2009-06-11T12:06:34+00:00 >>
 
+=head2 ISO8601StrictDateStr
+
+=head2 ISO8601StrictTimeStr
+
+=head2 ISO8601StrictDateTimeStr
+
+=head2 ISO8601StrictDateTimeTZStr
+
+As above, only in addition to validating the strings against regular
+expressions, an attempt is made to actually parse the data into a L<DateTime>
+object.  This will catch cases like C<< 2013-02-31 >> which look correct but do not
+correspond to real-world values.  Note that this bears a computation
+penalty.
+
 =head2 COERCIONS
 
 The date types will coerce from:
@@ -319,6 +401,8 @@ e.g.:-
 2012-01-13T170500Z => 2012-01-13T17:05:00Z
 20120113T17:05:00Z => 2012-01-13T17:05:00Z
 
+In addition, there are coercions from these string types to L<DateTime>.
+
 =back
 
 =head1 DURATION CONSTRAINTS
@@ -333,7 +417,7 @@ An ISO8601 time duration string. E.g. C<< PT01H01M01S >>
 
 =head2 ISO8601DateTimeDurationStr
 
-An ISO8601 comboined date and time duration string. E.g. C<< P01Y01M01DT01H01M01S >>
+An ISO8601 combined date and time duration string. E.g. C<< P01Y01M01DT01H01M01S >>
 
 =head2 COERCIONS
 
@@ -362,54 +446,12 @@ L<MooseX::Types::DateTime>.
 
 =back
 
-=head1 SEE ALSO
-
-=over
-
-=item *
-
-L<MooseX::Types::DateTime>
-
-=item *
-
-L<DateTime>
-
-=item *
-
-L<DateTime::Duration>
-
-=item *
-
-L<DateTime::Format::Duration>
-
-=back
-
-=head1 VERSION CONTROL
-
-    http://github.com/bobtfish/moosex-types-iso8601/tree/master
-
-Patches are welcome.
-
-=head1 SEE ALSO
-
-=over
-
-=item *
-
-http://en.wikipedia.org/wiki/ISO_8601
-
-=item *
-
-http://dotat.at/tmp/ISO_8601-2004_E.pdf
-
-=back
-
 =head1 FEATURES
 
 =head2 Fractional seconds
 
 If provided, the number of seconds in time types is represented to microsecond
-accuracy. A full stop character is used as the decimal seperator, which is
+accuracy. A full stop character is used as the decimal separator, which is
 allowed, but deprecated in preference to the comma character in
 I<ISO 8601:2004>.
 
@@ -419,7 +461,7 @@ Probably full of them, patches are very welcome.
 
 Specifically missing features:
 
-=over
+=over 4
 
 =item *
 
@@ -431,8 +473,7 @@ No week number type
 
 =item *
 
-"Basic format", which lacks seperator characters, is not supported for
-reading or writing.
+"Basic format", which lacks separator characters, is not supported for reading or writing.
 
 =item *
 
@@ -440,31 +481,73 @@ Tests are rubbish.
 
 =back
 
-=head1 AUTHOR
+=head1 SEE ALSO
 
-=over
+=over 4
 
-=item Tomas Doran (t0m) C<< <bobtfish@bobtfish.net> >>
 
-=item Dave Lambley C<< <davel@state51.co.uk> >>
 
 =back
 
+* L<MooseX::Types::DateTime>
+* L<DateTime>
+* L<DateTime::Duration>
+* L<DateTime::Format::ISO8601>
+* L<DateTime::Format::Duration>
+* L<http://en.wikipedia.org/wiki/ISO_8601>
+* L<http://dotat.at/tmp/ISO_8601-2004_E.pdf>
+
+=head1 ACKNOWLEDGEMENTS
+
 The development of this code was sponsored by my employer L<http://www.state51.co.uk>.
 
-=head2 Contributors
+=head1 AUTHORS
 
-=over
+=over 4
 
-=item Aaron Moses
+=item *
+
+Tomas Doran (t0m) <bobtfish@bobtfish.net>
+
+=item *
+
+Dave Lambley <davel@state51.co.uk>
 
 =back
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-    Copyright (c) 2009 Tomas Doran. Some rights reserved.
-    This program is free software; you can redistribute
-    it and/or modify it under the same terms as Perl itself.
+This software is copyright (c) 2009 by Tomas Doran.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=head1 CONTRIBUTORS
+
+=for stopwords Karen Etheridge Dave Lambley zebardy Aaron Moses Gregory Oschwald
+
+=over 4
+
+=item *
+
+Karen Etheridge <ether@cpan.org>
+
+=item *
+
+Dave Lambley <dave@lambley.me.uk>
+
+=item *
+
+zebardy <zebardy@gmail.com>
+
+=item *
+
+Aaron Moses <zebardy@gmail.com>
+
+=item *
+
+Gregory Oschwald <goschwald@maxmind.com>
+
+=back
 
 =cut
-
